@@ -31,11 +31,11 @@
 #
 
 """Apply JSON-Patches according to
-http://tools.ietf.org/html/draft-ietf-appsawg-json-patch-04"""
+http://tools.ietf.org/html/draft-ietf-appsawg-json-patch-05"""
 
 # Will be parsed by setup.py to determine package metadata
 __author__ = 'Stefan Kögl <stefan@skoegl.net>'
-__version__ = '0.5'
+__version__ = '0.6'
 __website__ = 'https://github.com/stefankoegl/python-json-patch'
 __license__ = 'Modified BSD License'
 
@@ -84,12 +84,12 @@ def apply_patch(doc, patch, in_place=False):
     :rtype: dict
 
     >>> doc = {'foo': 'bar'}
-    >>> other = apply_patch(doc, [{'add': '/baz', 'value': 'qux'}])
+    >>> other = apply_patch(doc, [{'op': 'add', 'path': '/baz', 'value': 'qux'}])
     >>> doc is not other
     True
     >>> other
     {'foo': 'bar', 'baz': 'qux'}
-    >>> apply_patch(doc, [{'add': '/baz', 'value': 'qux'}], in_place=True)
+    >>> apply_patch(doc, [{'op': 'add', 'path': '/baz', 'value': 'qux'}], in_place=True)
     {'foo': 'bar', 'baz': 'qux'}
     >>> doc == other
     True
@@ -125,23 +125,26 @@ class JsonPatch(object):
     """A JSON Patch is a list of Patch Operations.
 
     >>> patch = JsonPatch([
-    ...     {'add': '/foo', 'value': 'bar'},
-    ...     {'add': '/baz', 'value': [1, 2, 3]},
-    ...     {'remove': '/baz/1'},
-    ...     {'test': '/baz', 'value': [1, 3]},
-    ...     {'replace': '/baz/0', 'value': 42},
-    ...     {'remove': '/baz/1'},
+    ...     {'op': 'add', 'path': '/foo', 'value': 'bar'},
+    ...     {'op': 'add', 'path': '/baz', 'value': [1, 2, 3]},
+    ...     {'op': 'remove', 'path': '/baz/1'},
+    ...     {'op': 'test', 'path': '/baz', 'value': [1, 3]},
+    ...     {'op': 'replace', 'path': '/baz/0', 'value': 42},
+    ...     {'op': 'remove', 'path': '/baz/1'},
     ... ])
     >>> doc = {}
-    >>> patch.apply(doc)
-    {'foo': 'bar', 'baz': [42]}
+    >>> result = patch.apply(doc)
+    >>> expected = {'foo': 'bar', 'baz': [42]}
+    >>> result == expected
+    True
 
     JsonPatch object is iterable, so you could easily access to each patch
     statement in loop:
 
     >>> lpatch = list(patch)
-    >>> lpatch[0]
-    {'add': '/foo', 'value': 'bar'}
+    >>> expected = {'op': 'add', 'path': '/foo', 'value': 'bar'}
+    >>> lpatch[0] == expected
+    True
     >>> lpatch == patch.patch
     True
 
@@ -188,6 +191,18 @@ class JsonPatch(object):
     def __iter__(self):
         return iter(self.patch)
 
+
+    def __hash__(self):
+        return hash(frozenset(self.operations))
+
+
+    def __eq__(self, other):
+        if not isinstance(other, JsonPatch):
+            return False
+
+        return self.patch == other.patch
+
+
     @classmethod
     def from_string(cls, patch_str):
         """Creates JsonPatch instance from string source.
@@ -231,19 +246,19 @@ class JsonPatch(object):
                 for operation in compare_list(path, value, other):
                     yield operation
             else:
-                yield {'replace': '/'.join(path), 'value': other}
+                yield {'op': 'replace', 'path': '/'.join(path), 'value': other}
 
         def compare_dict(path, src, dst):
             for key in src:
                 if key not in dst:
-                    yield {'remove': '/'.join(path + [key])}
+                    yield {'op': 'remove', 'path': '/'.join(path + [key])}
                     continue
                 current = path + [key]
                 for operation in compare_values(current, src[key], dst[key]):
                     yield operation
             for key in dst:
                 if key not in src:
-                    yield {'add': '/'.join(path + [key]), 'value': dst[key]}
+                    yield {'op': 'add', 'path': '/'.join(path + [key]), 'value': dst[key]}
 
         def compare_list(path, src, dst):
             lsrc, ldst = len(src), len(dst)
@@ -254,10 +269,10 @@ class JsonPatch(object):
             if lsrc < ldst:
                 for idx in range(lsrc, ldst):
                     current = path + [str(idx)]
-                    yield {'add': '/'.join(current), 'value': dst[idx]}
+                    yield {'op': 'add', 'path': '/'.join(current), 'value': dst[idx]}
             elif lsrc > ldst:
                 for idx in reversed(range(ldst, lsrc)):
-                    yield {'remove': '/'.join(path + [str(idx)])}
+                    yield {'op': 'remove', 'path': '/'.join(path + [str(idx)])}
 
         return cls(list(compare_dict([''], src, dst)))
 
@@ -288,19 +303,24 @@ class JsonPatch(object):
         return obj
 
     def _get_operation(self, operation):
-        for action, op_cls in self.operations.items():
-            if action in operation:
-                location = operation[action]
-                return op_cls(location, operation)
+        if 'op' not in operation:
+            raise JsonPatchException("Operation does not contain 'op' member")
+            raise Exception
+
+        op = operation['op']
+        if op not in self.operations:
+            raise JsonPatchException("Unknown operation '%s'" % op)
+
+        cls = self.operations[op]
+        return cls(operation)
 
-        raise JsonPatchException("invalid operation '%s'" % operation)
 
 
 class PatchOperation(object):
     """A single operation inside a JSON Patch."""
 
-    def __init__(self, location, operation):
-        self.location = location
+    def __init__(self, operation):
+        self.location = operation['path']
         self.operation = operation
 
     def apply(self, obj):
@@ -346,6 +366,17 @@ class PatchOperation(object):
             return obj, part_variants[0]
 
 
+    def __hash__(self):
+        return hash(frozenset(self.operation.items()))
+
+
+    def __eq__(self, other):
+        if not isinstance(other, PatchOperation):
+            return False
+
+        return self.operation == other.operation
+
+
 class RemoveOperation(PatchOperation):
     """Removes an object property or an array element."""
 
@@ -407,8 +438,9 @@ class MoveOperation(PatchOperation):
     def apply(self, obj):
         subobj, part = self.locate(obj, self.location)
         value = subobj[part]
-        RemoveOperation(self.location, self.operation).apply(obj)
-        AddOperation(self.operation['to'], {'value': value}).apply(obj)
+
+        RemoveOperation({'op': 'remove', 'path': self.location}).apply(obj)
+        AddOperation({'op': 'add', 'path': self.operation['to'], 'value': value}).apply(obj)
 
 
 class TestOperation(PatchOperation):
@@ -434,4 +466,4 @@ class CopyOperation(PatchOperation):
     def apply(self, obj):
         subobj, part = self.locate(obj, self.location)
         value = copy.deepcopy(subobj[part])
-        AddOperation(self.operation['to'], {'value': value}).apply(obj)
+        AddOperation({'op': 'add', 'path': self.operation['to'], 'value': value}).apply(obj)
